package mschmidt.komo.datatools;

/*
 * Instances of this wizard object class control the database restore process.
 */

import java.io.File;
import java.util.ArrayList;

import mschmidt.komo.Application;
import mschmidt.komo.ApplicationImages;
import mschmidt.komo.KomoPlugin;
import mschmidt.komo.preferences.PreferenceConstants;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.wizard.Wizard;

public class RestoreWizard extends Wizard {

	// Instantiate the data model
	RestoreData data = new RestoreData();

	private IPreferenceStore store = KomoPlugin.getDefault()
			.getPreferenceStore();

	/*
	 * Constructor to create the wizard and initializes variables.
	 */
	public RestoreWizard() {
		String engineString = new String();
		setWindowTitle("Database Maintenance");
		setDefaultPageImageDescriptor(KomoPlugin
				.getImageDescriptor(ApplicationImages.DB_RESTORE32_ICON));

		// Create dialog settings if they don't exist, then set them.
		IDialogSettings dlgSettings = KomoPlugin.getDefault()
				.getDialogSettings().getSection("RestoreSection");
		if (dlgSettings == null) {
			dlgSettings = KomoPlugin.getDefault().getDialogSettings()
					.addNewSection("RestoreSection");
			dlgSettings.put(WizardConstants.RESTORE_ANALYZE, true);
			dlgSettings.put(WizardConstants.RESTORE_CLEAN, false);
			dlgSettings.put(WizardConstants.RESTORE_EXIT_ON_ERROR, false);
			dlgSettings.put(WizardConstants.RESTORE_FILE, "");
			dlgSettings.put(WizardConstants.RESTORE_IGNORE_VERSION, false);
			dlgSettings.put(WizardConstants.RESTORE_ITEMS, 0);
			dlgSettings.put(WizardConstants.RESTORE_VERBOSE, true);

			// Attempt to find the pg_restore file.
			engineString = getEngine(WizardConstants.RESTORE_ENGINE_APP);
			dlgSettings.put(WizardConstants.RESTORE_ENGINE, engineString);

			// Attempt to find the dropdb file.
			engineString = getEngine(WizardConstants.RESTORE_DROP_APP);
			dlgSettings.put(WizardConstants.RESTORE_DROP, engineString);
		}
		setDialogSettings(dlgSettings);

		/*
		 * If this wizard is initially opened before the JDBC is entered, the
		 * restore engine and drop utility may not be in the Wizard Constants.
		 * Try to find them if they aren't present.
		 */
		engineString = getDialogSettings().get(WizardConstants.RESTORE_ENGINE);
		if (engineString == null || engineString.length() == 0) {
			engineString = getEngine(WizardConstants.RESTORE_ENGINE_APP);
			if (engineString != null && engineString.length() > 0) {
				getDialogSettings().put(WizardConstants.RESTORE_ENGINE,
						engineString);
			}
		}

		engineString = getDialogSettings().get(WizardConstants.RESTORE_DROP);
		if (engineString == null || engineString.length() == 0) {
			engineString = getEngine(WizardConstants.RESTORE_DROP_APP);
			if (engineString != null && engineString.length() > 0) {
				getDialogSettings().put(WizardConstants.RESTORE_DROP,
						engineString);
			}
		}

		data.doAnalyze = getDialogSettings().getBoolean(
				WizardConstants.RESTORE_ANALYZE);
		data.doClean = getDialogSettings().getBoolean(
				WizardConstants.RESTORE_CLEAN);
		data.doExitOnError = getDialogSettings().getBoolean(
				WizardConstants.RESTORE_EXIT_ON_ERROR);
		data.dropUtil = getDialogSettings().get(WizardConstants.RESTORE_DROP);
		data.ignoreVersion = getDialogSettings().getBoolean(
				WizardConstants.RESTORE_IGNORE_VERSION);
		data.itemsSelection = getDialogSettings().getInt(
				WizardConstants.RESTORE_ITEMS);
		data.doVerbose = getDialogSettings().getBoolean(
				WizardConstants.RESTORE_VERBOSE);
		data.restoreEngine = getDialogSettings().get(
				WizardConstants.RESTORE_ENGINE);
		data.restoreFile = getDialogSettings()
				.get(WizardConstants.RESTORE_FILE);
	}

	/*
	 * (non-Javadoc) Add page to the wizard.
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	@Override
	public final void addPages() {
		RestoreWizardPage page = new RestoreWizardPage();
		addPage(page);
	}

	/*
	 * Try to get the restore applications @return the application or an empty
	 * string if it can't be found.
	 */
	private String getEngine(String appName) {
		String retString = new String();
		String tempString = store.getString(PreferenceConstants.JDBC_PATH);
		if (tempString != null && tempString.length() > 0) {
			File tempFile = new File(tempString);
			if (tempFile.isFile()) {
				tempString = tempFile.getParent();
				tempFile = new File(tempString);
				if (tempFile.isDirectory()) {
					tempString = tempFile.getParent();
					tempFile = new File(tempString);
					if (tempFile.isDirectory()) {
						tempString = tempString + File.separator + "bin"
								+ File.separator + appName;
						tempFile = new File(tempString);
						if (tempFile.isFile()) {
							retString = tempString;
						}
					}
				}
			}
		}
		return retString;
	}

	/*
	 * (non-Javadoc) Processing that occurs when the Finish button is pressed.
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		boolean allOK = true;

		// Save dialog settings
		if (getDialogSettings() != null) {
			getDialogSettings().put(WizardConstants.RESTORE_ANALYZE,
					data.doAnalyze);
			getDialogSettings()
					.put(WizardConstants.RESTORE_CLEAN, data.doClean);
			getDialogSettings()
					.put(WizardConstants.RESTORE_DROP, data.dropUtil);
			getDialogSettings().put(WizardConstants.RESTORE_EXIT_ON_ERROR,
					data.doExitOnError);
			getDialogSettings().put(WizardConstants.RESTORE_ENGINE,
					data.restoreEngine);
			getDialogSettings().put(WizardConstants.RESTORE_FILE,
					data.restoreFile);
			getDialogSettings().put(WizardConstants.RESTORE_IGNORE_VERSION,
					data.ignoreVersion);
			getDialogSettings().put(WizardConstants.RESTORE_ITEMS,
					data.itemsSelection);
			getDialogSettings().put(WizardConstants.RESTORE_VERBOSE,
					data.doVerbose);
		}

		// Make sure that the restore file is a valid file and if so, do job
		File file = new File(data.restoreFile);
		if (!file.isFile()) {
			MessageDialog
					.openWarning(
							getShell(),
							"Restore Warning",
							"The selected restore file is not a valid file\nor could not be located.\n\nRestore operation not performed.");
			allOK = false;
		}
		if (allOK) {
			// Check for password and then run restore job
			if (Application.dbPassword.checkPassword()) {
				ArrayList<String> parmList = new ArrayList<String>();
				boolean doDrop = false;
				int promptNum;

				// Required settings.
				parmList.add(data.restoreEngine);

				parmList.add("--host="
						+ store.getString(PreferenceConstants.DB_URL));

				parmList.add("--port="
						+ store.getString(PreferenceConstants.DB_PORT));

				parmList.add("--user="
						+ store.getString(PreferenceConstants.DB_USER));

				parmList.add("-W");

				// Optional settings.
				switch (data.itemsSelection) {
				case 0: // The delete and replace option.
					parmList.add("-C");
					doDrop = true;
					break;
				case 1: // The Create database first option.
					parmList.add("-C");
					break;
				case 3: // The data only option.
					parmList.add("-a");
					break;
				case 4: // The data only, disable triggers option.
					parmList.add("-a");
					parmList.add("--disable-triggers");
					break;
				case 5: // The schema only option.
					parmList.add("-s");
				}

				if (data.ignoreVersion) {
					parmList.add("-i");
				}

				if (data.doVerbose) {
					parmList.add("-v");
				}

				if (!doDrop) {
					if (data.doClean) {
						parmList.add("-c");
					}

					if (data.doExitOnError) {
						parmList.add("-e");
					}

				}

				if (data.itemsSelection == 0 || data.itemsSelection == 1) {
					parmList.add("--dbname=postgres");
					promptNum = 2;
				} else {
					parmList.add("--dbname="
							+ store.getString(PreferenceConstants.DB_NAME));
					promptNum = 1;
				}

				parmList.add(data.restoreFile);

				// place the list in an array to pass to the job.
				int listSize = parmList.size();
				String[] parmArray = new String[listSize];
				for (int k = 0; k < listSize; ++k) {
					parmArray[k] = parmList.get(k);
				}
				// Create and schedule the job.
				String appString = null;
				if (doDrop) {
					appString = data.dropUtil;
				}
				if (data.itemsSelection == 0 || data.itemsSelection == 1) {
					String message = "This operation will delete the database and replace it\nwith the archived database in the restore file.\nOnce deleted, the database cannot be recovered.\nThe restore file must contain an intact, full database backup.\n\nAre you sure you want to proceed?";
					allOK = MessageDialog.openQuestion(getShell(), "Confirm Restore", message); 
				}
				if (allOK) {
					Job restoreJob = new UtilityProcessJob("Restore",
						ApplicationImages.DB_RESTORE_ICON, parmArray,
						appString, promptNum, data.doAnalyze);
					restoreJob.schedule();
				}
			}
		}

		return allOK;
	}

}
